{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWBeveles.pas                                          }
{ Short description  :                                                         }
{ Author             : Erol S. Uzuner                                          }
{ E-Mail             : sarcon@tzi.de                                           }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 09.09.96                                                }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : Copyright (C) 1996-1998                                 }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          : TDFWBeveles                                               }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWBeveles;

interface

uses
  SysUtils, WinTypes,  Classes, Graphics, Controls,
  ExtCtrls;

const
      ComponentVersion='1.0';
      ComponentAuthor='Erol S. Uzuner';
      ComponentEMail='sarcon@tzi.de';
      ComponentHomePage='';

type

  TDFWBeveles = class(TBevel)
  private
    { Private-Deklarationen }
    FBorderWidth : Byte;
    FStyle: TBevelStyle;
    FCaption : TCaption;
    procedure SetStyle(Value: TBevelStyle);
    procedure SetCaption(Value : TCaption);
    procedure SetBorderWidth(Value : Byte);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
  published
    { Published-Deklarationen }
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property Style: TBevelStyle read FStyle write SetStyle default bsLowered;
    property Caption: TCaption read FCaption write SetCaption;
    property Font;
    property BorderWidth : Byte read FBorderWidth write SetBorderWidth;
    property Shape;
  end;

implementation

constructor TDFWBeveles.Create(AOwner : TComponent);
begin
    inherited Create(AOwner);
    FBorderWidth := 0;
    Text := '';
end;

procedure TDFWBeveles.Paint;
var
  H: Integer;
  R: TRect;
  C: array [Byte] of Char;
  CLen: Integer;
  Color1, Color2, Temp : TColor;
  CaptionExists : Boolean;


  procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Color := C;
      MoveTo(X1, Y1);
      LineTo(X2, Y2);
    end;
  end;

  procedure DrawBevelText;
  begin
       with Canvas do
       begin
           StrPCopy(C, Text);
           if C[0] <> #0 then
           begin
             StrPCopy(C, Text);
             CLen := StrLen(C);
             R := Rect(8, 0, 0, H);
             DrawText(Canvas.Handle, C, CLen, R, DT_LEFT or DT_SINGLELINE or DT_CALCRECT);
             Brush.Style := bsClear;
             DrawText(Canvas.Handle, C, CLen, R, DT_LEFT or DT_SINGLELINE);
           end;
       end;
  end;

  procedure BevelBox;
  begin
     with Canvas do
     begin
          Pen.Color := Color1;

          if CaptionExists then begin
              PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                        Point(R.Left + 6, R.Top)]);
              PolyLine([Point(R.Left + TextWidth(Self.Text)+ 12, R.Top), Point(R.Right,R.Top)])
           end else begin
              PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                        Point(R.Left, R.Top)]);
              PolyLine([Point(R.Left, R.Top), Point(R.Right,R.Top)])
           end;

           Dec(R.Bottom);
           Dec(R.Right);
           Pen.Color := Color2;
           PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
                     Point(R.Left, R.Bottom)]);
     end;
  end;

  procedure BevelFrame;
  begin
     with Canvas do
     begin
          Pen.Color := Color1;

          Dec(R.Bottom,2);
          Dec(R.Right);
          if CaptionExists then begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                     Point(R.Left + 6, R.Top)]);
            PolyLine([Point(R.Left + TextWidth(Self.Text)+ 6, R.Top), Point(R.Right,R.Top)]);
          end else begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                     Point(R.Left, R.Top)]);
            PolyLine([Point(R.Left, R.Top), Point(R.Right,R.Top)]);
          end;

          Dec(R.Right);
          PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
                     Point(R.Left, R.Bottom)]);

          Pen.Color := Color2;
          Inc(R.Left);
          Inc(R.Top);
          Dec(R.Bottom);
          Dec(R.Right);
          if CaptionExists then begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                       Point(R.Left + 6, R.Top)]);
            PolyLine([Point(R.Left + TextWidth(Self.Text)+ 6, R.Top), Point(R.Right,R.Top)]);
          end else begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                       Point(R.Left, R.Top)]);
            PolyLine([Point(R.Left, R.Top), Point(R.Right,R.Top)]);
          end;

          Dec(R.Left);
          Inc(R.Bottom,2);
          Inc(R.Right,2);
          PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
                     Point(R.Left, R.Bottom)]);
     end;
  end;

  procedure BevelInnerBox;
  begin
       if FBorderWidth > 0 then
       begin
         with canvas do
         begin
             Temp := Color1;
             Color1 := Color2;
             Color2 := Temp;
             Inc(R.Left, FBorderWidth);
             Inc(R.Top, FBorderWidth);
             Dec(R.Right, FBorderWidth);
             Dec(R.Bottom, FBorderWidth);
             BevelBox;
          end;
       end;
  end;

begin
  if Caption <> '' then CaptionExists := true
  else CaptionExists := false;

  with Canvas do
  begin
    Pen.Width := 1;
    if CaptionExists then begin
      Font := Self.Font;
      H := TextHeight('0');
      R := Rect(0, (H Div 2) - 1 , Width, Height);
    end else
      R := Rect(0, 0, Width, Height);

    if FStyle = bsLowered then
    begin
      Color1 := clBtnShadow;
      Color2 := clBtnHighlight;
    end
    else
    begin
      Color1 := clBtnHighlight;
      Color2 := clBtnShadow;
    end;

    case Shape of
      bsBox:
        begin
           BevelBox;
           Inc(R.Bottom);
           Inc(R.Right);
           BevelInnerBox;
           if CaptionExists then DrawBevelText;
        end;
      bsFrame:
        begin
          BevelFrame;
          Inc(R.Left);
          BevelInnerBox;
          if CaptionExists then DrawBevelText;
        end;
      bsTopLine:
        begin
          BevelLine(Color1, 0, 0, Width, 0);
          BevelLine(Color2, 0, 1, Width, 1);
          if CaptionExists then DrawBevelText;
        end;
      bsBottomLine:
        begin
          BevelLine(Color1, 0, Height - 2, Width, Height - 2);
          BevelLine(Color2, 0, Height - 1, Width, Height - 1);
          if CaptionExists then DrawBevelText;
        end;
      bsLeftLine:
        begin
          BevelLine(Color1, 0, 0, 0, Height);
          BevelLine(Color2, 1, 0, 1, Height);
          if CaptionExists then DrawBevelText;
        end;
      bsRightLine:
        begin
          BevelLine(Color1, Width - 2, 0, Width - 2, Height);
          BevelLine(Color2, Width - 1, 0, Width - 1, Height);
          if CaptionExists then DrawBevelText;
        end;
    end;
  end;
end;

procedure TDFWBeveles.SetStyle(Value: TBevelStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TDFWBeveles.SetCaption(Value: TCaption);
begin
    if FCaption <> Value then
    begin
      FCaption := Value;
      Text := Value;
      Invalidate;
    end;
end;

procedure TDFWBeveles.SetBorderWidth(Value : Byte);
begin
    if FBorderWidth <> Value then
    begin
       FBorderWidth := Value;
       Invalidate;
    end;
end;

function TDFWBeveles.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWBeveles.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWBeveles.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWBeveles.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
