{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }
{                                                                              }
{ Filename           : DFWEdites.pas                                           }
{ Short description  :                                                         }
{ Author             : Erol S. Uzuner                                          }
{ E-Mail             : sarcon@tzi.de                                           }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 26.03.99                                                }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : Copyright (C) 1996-2000                                 }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   : 28.08.97 Initial release                                }
{                      28.08.97 berarbeitung                                  }
{                      31.08.97 Einfhrung der TEdites Klasse                  }
{                      02.08.97 Button eingefgt in TEdites                    }
{                      11.09.97 Einfrbungen bei Aktivierung eingefgt         }
{                      22.09.97 Bug beseitigt, nun  auch bei Focuserhalt       }
{                               aktive Einfrbung                              }
{                      23.09.97 Multiline Bug behoben, nun wird vk_return      }
{                               nicht mehr weitergeleitet Keys property        }
{                               eingefhrt.                                    }
{                      25.09.97 Frbungsfehler aufgehoben                      }
{                      03.02.99 LightEdites eingefgt                          }
{                      26.03.99 Fehler mit den Farben behoben                  }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWLightEdites                                           }
{ Classes          : TDFWEditesEdit                                            }
{ Types            : TEditStyle, TDataKind, TGlyphStyle, TSetItemIndex,        }
{                    TKeySet, TKeySets                                         }
{ Constants        : DEF_FOCUSED_COLOR, DEF_REQUIRED_COLOR                     }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWEdites;

interface

uses
  Windows, Messages,  Graphics, Classes, Controls, Forms, SysUtils,
  ExtCtrls, StdCtrls, Menus, Buttons, DFWGlobes, Dialogs;

const
   DEF_FOCUSED_COLOR = clAqua;
   DEF_REQUIRED_COLOR = clYellow;

   ComponentVersion='1.0';
   ComponentAuthor='Erol S. Uzuner';
   ComponentEMail='sarcon@tzi.de';
   ComponentHomePage='';

type
  TEditStyle = (esUnderline, esCtl2D, esCtl3D);
  TDataKind = (dkCustom, dkDate, dkTime, dkInteger, dkFloat);
  TGlyphStyle = (gsCustom, gsCalendar, gsEllipsis, gsDown, gsDown3D, gsArrow);
  TSetItemIndex = procedure(Sender : TObject; Var Value : Longint) of object;
  TKeySet = (ksReturn, ksEscape);
  TKeySets = set of TKeySet;


  TDFWEditesEdit = class(TEdit)
   private
    { Private-Deklarationen }
    FCanvas: TControlCanvas;
    FLineStyle: TBevelStyle;
    FEditStyle : TEditStyle;
    FAlignment: TAlignment;
    FRequired : Boolean;
    FFocusedColor : TColor;
    FRequiredColor : TColor;
    FBeforeFocusColor : TColor;
    FDataKind : TDataKind;
    FKeys: TKeySets;
    FYear2000 : Integer;
    FFirstCharUpperCase : Boolean;

    procedure FocusColorize(GotFocus : boolean);
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
    procedure WMKillFocus(var Message: TWMSetFocus); message WM_KILLFOCUS;
    procedure CMExit(var Message : TCMExit); message CM_Exit;
    procedure WMKeyUp(var Message : TWMKeyUp); message WM_KEYUP;
   procedure CMTextChanged(var Message : TMessage); message CM_TEXTCHANGED;

  protected
    { Protected-Deklarationen }
    procedure SetEditStyle(Value: TEditStyle);
    procedure SetLineStyle(Value: TBevelStyle);
    procedure SetAlignment(Value : TAlignment);
    procedure SetRequired(value : Boolean);
    procedure SetFRColor(index : Integer; Value : TColor);
    procedure CreateParams(var Params: TCreateParams); override;
    procedure KeyPress(var Key: Char); override;
    procedure SetYear2000(value :Integer);
    procedure setFirstCharUpper(value : boolean);
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure RequiredColorize;
{$IFNDEF D2}
    procedure Clear; override;
{$ELSE}
    procedure Clear;
{$ENDIF}
    property Alignment : TAlignment read FAlignment write SetAlignment default taLeftJustify;
    property DataKind : TDataKind read FDataKind write FDataKind default dkCustom;
    property EditStyle: TEditStyle read FEditStyle write SetEditStyle default esCtl3D;
    property FirstCharUpperCase : Boolean read FFirstCharUpperCase write setFirstCharUpper default false;
    property LineStyle: TBevelStyle read FLineStyle write SetLineStyle default bsRaised;
    property Required : Boolean read FRequired write SetRequired default false;
    property FocusedColor : TColor index 1 read FFocusedColor write SetFRColor default DEF_FOCUSED_COLOR;
    property RequiredColor : TColor index 2 read FRequiredColor write SetFRColor default DEF_REQUIRED_COLOR;
    property Keys: TKeySets read FKeys write FKeys;
    property Year2000 : Integer read FYear2000 write SetYear2000 default 90;
  end;

  TDFWLightEdites = class(TDFWEditesEdit)
  private
    FDataField : String;

    FOnWrite : TNotifyEvent;
    FOnRead : TNotifyEvent;

  protected
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
  public
    procedure Write;
    procedure Read;
    procedure DoPopupComplete(var Key : Char);

  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property Alignment;
    property DataKind;
    property EditStyle;
    property FirstCharUpperCase;
    property LineStyle;
    property Required;
    property FocusedColor;
    property RequiredColor;
    property Keys;
    property Year2000;

    property DataField : String read FDataField write FDataField;
    property OnRead : TNotifyEvent read FOnRead write FOnRead;
    property OnWrite : TNotifyEvent read FOnWrite write FOnWrite;

  end;

implementation

{$R DFWEdites.Res}

constructor TDFWEditesEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLineStyle := bsRaised;
  FEditStyle := esCtl3D;
  FCanvas := NIL;
  FDataKind := dkCustom;
  FRequired := false;
  FFirstCharUpperCase := false;
  FRequiredColor := DEF_REQUIRED_COLOR;
  FFocusedColor := DEF_FOCUSED_COLOR;
  FBeforeFocusColor := Color;
  FAlignment := taLeftJustify;
  FYear2000 := 90;
end;

destructor TDFWEditesEdit.Destroy;
begin
  FCanvas.Free;
  inherited Destroy;
end;

procedure TDFWEditesEdit.RequiredColorize;
begin
 if FRequired then begin
    if (Text = '') then begin
       if Color <> FRequiredColor then Color := FRequiredColor;
    end else begin
      Color := FBeforeFocusColor;
      if FEditStyle = esUnderline then ParentColor := true
    end;
  end;
end;

procedure TDFWEditesEdit.Clear;
begin
    inherited Clear;
    RequiredColorize;
end;

procedure TDFWEditesEdit.setFirstCharUpper(value : Boolean);
begin
   if (value <> FFirstCharUpperCase) then
     FFirstCharUpperCase := value;
end;

procedure TDFWEditesEdit.SetAlignment(Value : TAlignment);
begin
 if value <> FAlignment then
  begin
    FAlignment := value;
    RecreateWnd;
  end;
end;

procedure TDFWEditesEdit.KeyPress(var Key: Char);

  procedure CheckNumber;
  begin
     {Nummern}
     if Key > Chr(31) then begin
        try  StrToInt(Key)
        except Key := Chr(0);
        end;
     end;
  end;

var str : String;

begin
 if ksReturn in Keys then
  begin
    if Key = Chr(Vk_Return) then
    begin
      Key := Chr(0);
      (Owner.Owner as TControl).Perform(wm_NextDlgCtl,0,0);
    end;
  end;
  if ksEscape in Keys then
  begin
    if Key = Chr(Vk_Escape) then
    begin
      Key := Chr(0);
     (Owner.Owner as TControl).Perform(wm_NextDlgCtl,1,0);
    end;
  end;

  if Key = Chr(Vk_Return) then Key := Chr(0);

  case FDataKind of
     dkCustom :
       begin
          if FFirstCharUpperCase then  begin
             if Length(Text) = 0 then begin
               str := '' + Key;
               Key := UpperCase(str)[1];
             end;
          end;
       end;
     dkDate :
       begin
         if (Key = DateSeparator) then begin
             if (Text[Length(Text)] = DateSeparator) then
               Key := Chr(0);
         end else CheckNumber;
       end;
     dkFloat :   ;
     dkInteger : CheckNumber;
     dkTime :    ;
  end;

  inherited KeyPress(Key);
end;

procedure TDFWEditesEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  case Alignment of
    taLeftJustify  : Params.Style := Params.Style or (ES_LEFT OR ES_MULTILINE);
    taRightJustify : Params.Style := Params.Style or (ES_RIGHT OR ES_MULTILINE);
    taCenter       : Params.Style := Params.Style or (ES_CENTER OR ES_MULTILINE);
  end;
end;

procedure TDFWEditesEdit.FocusColorize(GotFocus : boolean);
begin
  if GotFocus then begin
     if Color <> FRequiredColor then FBeforeFocusColor := Color;
     if Color <> FFocusedColor then Color := FFocusedColor;
  end else begin
     if FRequired AND (Text = '') then begin
         if Color <> FRequiredColor then Color := FRequiredColor;
     end else begin
        Color := FBeforeFocusColor;
        if (FEditStyle = esUnderline) then ParentColor := True
        {else if Color <> clWindow then Color :=  clWindow;}
     end;
  end;
end;

procedure TDFWEditesEdit.WMSetFocus(var Message: TWMSetFocus);
begin
  inherited;
  FocusColorize(true);
end;

procedure TDFWEditesEdit.WMKillFocus(var Message: TWMSetFocus);
begin
    inherited;
    FocusColorize(false);
end;

procedure TDFWEditesEdit.CMExit(var Message : TCMExit);
var
   doExit : boolean;
   str : String;
   Year, Month, Day : Word;
   strd : String;
begin
  doExit := true;
  if Text <> '' then begin
     case FDataKind of
       dkCustom :  doExit := true;
       dkDate : begin
           doExit := IsDate(Text);
           if doExit AND ((Length(Text) = 8) OR (Length(Text) = 7)) then begin
               str := Copy(Text, 7, 2);
               if Length(str) = 1 then str := '0' + str;
               DecodeDate(Date, Year, Month, Day);
               Year := Year - FYear2000;
               if (StrToInt(str) + 1900)  < Year then begin
                 // Jahr 2000
                 strd := Copy(Text, 0, 6) + '20' + str;
               end else begin
                // Jahr 1900
                strd := Copy(Text, 0, 6) + '19' + str;
               end;
               Text := strd;
           end;
        end;
       dkFloat :   doExit := IsAFloat(Text);
       dkInteger : doExit := IsAInt(Text);
       dkTime :    doExit := IsTime(Text);
     end;
  end;
  if doExit then inherited
  else begin
    Dialogs.ShowMessage('Ungltige Eingabe!');
    SetFocus;
  end;
end;

procedure TDFWEditesEdit.WMKeyUp(var Message: TWMKeyUp);
var ls : Integer;
begin
   case FDataKind of
     dkCustom : ;
     dkDate :
      begin
        if (Message.CharCode <> Vk_Back) AND (Message.CharCode <> vk_Delete) then
           case Length(Text) of
            2, 5:
               begin
                   ls := Length(Text);
                   if Text[ls] = DateSeparator then begin
                       case ls of
                         2 : Text := '0' + Text;
                         5 : Text := Copy(Text, 0, 3) + '0' + Copy(Text, 4, 5);
                       end;
                   end else Text := Text + DateSeparator;
                   SelStart := Length(Text) + 1;
                   SelLength := 0;
               end;
           end;
       end;
     dkFloat :   ;
     dkInteger : ;
     dkTime :    ;
   end;
   inherited;
end;

procedure TDFWEditesEdit.CMTextChanged(var Message : TMessage);
begin RequiredColorize;
end;

{*
procedure TDFWEditesEdit.WMSetText(var Message : TWMSetText);
begin RequiredColorize;
end;
*}

procedure TDFWEditesEdit.WMPaint(var Message: TWMPaint);

  procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with FCanvas do
    begin
      Pen.Color := C;
      MoveTo(X1, Y1);
      LineTo(X2, Y2);
    end;
  end;

  procedure Underlined;
  var
     Color1, Color2: TColor;
  begin
     if FCanvas = nil then
     begin
       FCanvas := TControlCanvas.Create;
       FCanvas.Control := Self;
     end;

      with FCanvas do
      begin
         if FLineStyle = bsLowered then
         begin
           Color1 := clBtnShadow;
           Color2 := clBtnHighlight;
         end
         else
         begin
           Color1 := clBtnHighlight;
           Color2 := clBtnShadow;
         end;

         BevelLine(Color1, 0, Height - 2, Width, Height - 2);
         BevelLine(Color2, 0, Height - 1, Width, Height - 1);
      end;
  end;

begin
  inherited;
  if FEditStyle = esUnderline then Underlined;
end;



procedure TDFWEditesEdit.SetEditStyle(Value: TEditStyle);
begin
  if value <> FEditStyle then
  begin
       FEditStyle := Value;
       case Value of
         esUnderline : begin
                       Ctl3D := false; BorderStyle := bsNone;
                       ParentColor := true;
                       end;
         esCtl2D   :   begin
                       Ctl3D := false; BorderStyle := bsSingle;
                       ParentColor := False;
                       Color := FBeforeFocusColor;
                       end;
         esCtl3D :     begin
                       Ctl3D := true;  BorderStyle := bsSingle;
                       ParentColor := False;
                       Color := FBeforeFocusColor;
                       end;
       end;
       Invalidate;
  end;
end;

procedure TDFWEditesEdit.SetLineStyle(Value: TBevelStyle);
begin
  if FEditStyle = esUnderline then
  begin
     if Value <> FLineStyle then
     begin
       FLineStyle := Value;
       Invalidate;
     end;
  end;
end;

procedure TDFWEditesEdit.SetRequired(Value : Boolean);
begin
     if FRequired <> Value then
     begin
         FRequired := Value;
         if Value AND (Text = '') then
         begin
            if Color <> FRequiredColor then Color := FRequiredColor;
         end else if Not(Value) then
         begin
           if ParentColor then Perform(CM_PARENTCOLORCHANGED, 0, 0)
           else if Color <> clWindow then Color :=  FBeforeFocusColor;
         end;
     end;
end;

procedure TDFWEditesEdit.SetFRColor(index : Integer; Value : TColor);
begin
     case index of
        1 :  if FFocusedColor <> Value then
             begin
                  FFocusedColor := Value; Invalidate;
                  IF Focused then Color := Value;
             end;
        2 :  if FRequiredColor <> Value then
             begin
                  FRequiredColor := Value;
                  if (Text = '') AND FRequired then Color := Value;
             end;
       end;

end;

procedure TDFWEditesEdit.SetYear2000(value : Integer);
begin FYear2000 := value;
end;

(*----------------------------------
 |
 *----------------------------------*)
procedure TDFWLightEdites.Write;
begin  If Assigned(FOnWrite) then FOnWrite(Self);
end;

procedure TDFWLightEdites.Read;
begin  If Assigned(FOnRead) then FOnRead(Self);
end;


procedure TDFWLightEdites.DoPopupComplete(VAR Key : Char);
{
  Hier wird completition durchgefhrt. Wenn einem TEdites Feld ein
  DropdownMenu zugewiesen wurde, wird nachgeschaut die Eingabe
  mit dem Auswahlmenu bereinstimmt, falls ja wird die Eingabe zulssig
}
var
   str : string;
   i, k : Integer;
begin
 if Key > #31 then begin
  if PopupMenu <> NiL then begin
    {Das ist der String + dem Zeichen }
    str := UpperCase(Copy(Text, 0, Length(Text) - SelLength) + Key);

    for i := 0 to PopupMenu.Items.Count -1 do begin
       if Pos(str, UpperCase(PopupMenu.Items[i].Caption)) = 1 then begin
               k := Length(Text) - SelLength;
               Text := PopupMenu.Items[i].Caption;
               SelStart := k+1;
               SelLength := Length(Text) - (k+1);
               Key := #0;
               Exit;
       end;
    end;
    Key := #0; {wenn nichts bereinstimmt wurde eine Falsche Taste gedrckt}
  end;
 end;
end;


function TDFWLightEdites.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWLightEdites.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWLightEdites.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWLightEdites.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.
