{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                    
{                                                                              }
{ Filename           : DFWPaneles.pas                                          }
{ Short description  : used by TDFWFoldPanel                                   }
{ Author             : Erol S. Uzuner                                          }
{ E-Mail             : sarcon@tzi.de                                           }
{ Internet           :                                                         }
{ Version            : 1.0                                                     }
{ Date last modified : 26.05.98                                                }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     : Copyright (C) 1996-1998                                 }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              : See helpfile                                            }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       :                                                           }
{ Classes          : TDFWPaneles                                               }
{ Types            :                                                           }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWPaneles;

interface

uses
  Windows, Messages, Controls, Classes, Graphics, ExtCtrls, DFWBitmapHolder;

type
  TDFWPaneles = class(TPanel)
  private
    { Private-Deklarationen }
    FBitmap : TBitmap;
    FBitmapHolder : TDFWBitmapHolder;
    FImageChangeLink: TDFWLinkChange;
    procedure WMEraseBkgnd(var MSG: TWMEraseBkgnd); message wm_EraseBkgnd;  {Hintergund zeichnen}
    procedure DrawBackground(TheHandle : THandle);
  protected
    { Protected-Deklarationen }
    function GetBitmap : TBitmap;
    procedure SetBitmap(Value : TBitmap);
    function GeTDFWBitmapHolder : TDFWBitmapHolder;
    procedure SeTDFWBitmapHolder(Value : TDFWBitmapHolder);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure DrawBorder;
    procedure Paint; override;
    procedure BitmapHolderChange(Sender: TObject);
  public
    { Public-Deklarationen }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
  published
    { Published-Deklarationen }
    property Bitmap : TBitmap read GetBitmap write SetBitmap;
    property BitmapHolder : TDFWBitmapHolder read GeTDFWBitmapHolder write SeTDFWBitmapHolder;
  end;


implementation

constructor TDFWPaneles.Create(AOwner : TComponent);
begin
     inherited Create(AOwner);
     FBitmap := TBitmap.Create;
     FBitmapHolder := NiL;
     FImageChangeLink := TDFWLinkChange.Create;
     FImageChangeLink.OnChange := BitmapHolderChange;
end;

destructor TDFWPaneles.Destroy;
begin
     FBitmap.Free;
     if FBitmapHolder <> NIL then
       FBitmapHolder.UnRegisterChanges(FImageChangeLink);
     FImageChangeLink.Free;
     inherited Destroy;
end;

procedure TDFWPaneles.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if Assigned(FBitmapHolder) AND (Operation = opRemove) AND (AComponent = FBitmapHolder) then
     FBitmapHolder := NIL;
end;


procedure TDFWPaneles.WMEraseBkgnd(var MSG: TWMEraseBkgnd);
begin
  inherited;
  if (FBitmap.Empty) AND (NoT(Assigned(FBitmapHolder)) OR
       (Assigned(FBitmapHolder) AND (FBitmapHolder.isEmpty))) then begin
       Canvas.Brush.Color := Color;
       Windows.FillRect(MSG.DC, GetClientRect, Canvas.Brush.Handle);
       DrawBorder;
  end else
    DrawBackground(MSG.DC);
end;

procedure DrawTiled(DC : HDC; CR: TRect; Bitmap : TBitmap);
var
  Row, Col : Integer;
  NumRows, NumCols: Integer;
  DrawX, DrawY, DrawWidth, DrawHeight: Integer;
  OldRect : TRect;
begin

  NumRows := CR.Bottom DIV Bitmap.Height;
  NumCols := CR.Right DIV Bitmap.Width;
  for Row := 0 to NumRows+1 do
    for Col := 0 to NumCols+1  do
    begin
      DrawX :=  (Col * Bitmap.Width) + CR.Left;
      DrawY :=  (Row * Bitmap.Height) + CR.Top;
      if (DrawX + Bitmap.Width) > CR.Right then
         DrawWidth :=  CR.Right - DrawX
      else
         DrawWidth :=  Bitmap.Width;

      if (DrawY + Bitmap.Height) > CR.Bottom then
         DrawHeight :=  CR.Bottom - DrawY
      else
         DrawHeight := Bitmap.Height;
      GetClipBox(DC, OldRect);
      IntersectCliprect(DC, CR.Left, CR.Top, CR.Right, CR.Bottom);
      BitBlt( DC,
              DrawX, DrawY, DrawWidth, DrawHeight,
              Bitmap.Canvas.Handle,
              0, 0, SRCCOPY);
      IntersectCliprect(DC, OldRect.Left, OldRect.Top, OldRect.Right, OldRect.Bottom);
    end;
end;


procedure TDFWPaneles.DrawBackground(TheHandle : THandle);
var Rect : TRect;
//    bmp : TBitmap;
begin
  Rect := GetClientRect;
  Rect.TopLeft := ClientRect.TopLeft;
  Rect.BottomRight := ClientRect.BottomRight;

  if NoT(FBitmap.Empty) then
      DrawTiled(TheHandle, rect, FBitmap)
  else if Assigned(FBitmapHolder) AND (NoT FBitmapHolder.isEmpty) then
    FBitmapHolder.TiledOnH(rect, TheHandle);

  {
  if (Assigned(bmp)) then begin
  (*  if BevelInner <> bvNone then
      InflateRect(rect, -BevelWidth, -BevelWidth);
    if BevelOuter <> bvNone then
      InflateRect(rect, -BevelWidth, -BevelWidth);
  *)
  end;
  }
  DrawBorder;
end;

procedure TDFWPaneles.DrawBorder;
var
  Rect: TRect;
  TopColor, BottomColor: TColor;

  procedure AdjustColors(Bevel: TPanelBevel);
  begin
    TopColor := clBtnHighlight;
    if Bevel = bvLowered then TopColor := clBtnShadow;
    BottomColor := clBtnShadow;
    if Bevel = bvLowered then BottomColor := clBtnHighlight;
  end;
begin
  Rect := GetClientRect;
  if BevelOuter <> bvNone then begin
    AdjustColors(BevelOuter);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;

  Frame3D(Canvas, Rect, Color, Color, BorderWidth);
  if BevelInner <> bvNone then begin
    AdjustColors(BevelInner);
    Frame3D(Canvas, Rect, TopColor, BottomColor, BevelWidth);
  end;
end;

procedure TDFWPaneles.Paint;
var
  Rect: TRect;
  FontHeight: Integer;
const
  Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);

begin
  Rect := GetClientRect;
  with Canvas do begin
    Brush.Color := Color;
    if (FBitmap.Empty) AND (NoT(Assigned(FBitmapHolder)) OR
       (Assigned(FBitmapHolder) AND (FBitmapHolder.isEmpty))) then
       FillRect(Rect)
    else
      DrawBackground(Canvas.handle);

    DrawBorder;
    Brush.Style := bsClear;
    Font := Self.Font;
    FontHeight := TextHeight('W');
    with Rect do begin
      Top := ((Bottom + Top) - FontHeight) div 2;
      Bottom := Top + FontHeight;
    end;
    DrawText(Handle, PChar(Caption), -1, Rect, (DT_EXPANDTABS or
      DT_VCENTER) or Alignments[Alignment]);
  end;

end;

function TDFWPaneles.GetBitmap : TBitmap;
begin Result := FBitmap;
end;

procedure TDFWPaneles.SetBitmap(Value : TBitmap);
begin
   FBitmap.Assign(Value);
   if FBitmapHolder <> nil then begin
    if Value <> NIL then
      FBitmapHolder.UnRegisterChanges(FImageChangeLink)
    else
      FBitmapHolder.RegisterChanges(FImageChangeLink);
   end;
   inherited Invalidate;
end;

function TDFWPaneles.GeTDFWBitmapHolder : TDFWBitmapHolder;
begin Result := FBitmapHolder;
end;

procedure TDFWPaneles.SeTDFWBitmapHolder(Value : TDFWBitmapHolder);
begin
  if FBitmapHolder <> nil then FBitmapHolder.UnRegisterChanges(FImageChangeLink);
  FBitmapHolder := Value;
  if FBitmapHolder <> nil then
  begin
     FBitmapHolder.RegisterChanges(FImageChangeLink);
     Value.FreeNotification(Self);
  end;
  invalidate;
end;

procedure TDFWPaneles.BitmapHolderChange(Sender: TObject);
begin inherited invalidate;
end;

end.
