{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWSTColors.pas                                         }
{ Short description  :                                                         }
{ Author             : Jakub Dusek                                             }
{ E-Mail             : j.dusek@seznam.cz                                       }
{ Internet           : http://dusek.zde.cz                                     }
{ Version            : 1.11                                                    }
{ Date last modified : 20.12.1999                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWSTColorsCombo, TDFWSTColorsList                       }
{ Classes          :                                                           }
{ Types            :                                                           }
{ Constants        : StdColors, SysColors                                      }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSTColors;

{$IFNDEF VER80}
 {$IFNDEF VER90}
  {$IFNDEF VER93}
    {$IFNDEF VER100}
      {$IFNDEF VER110}
        {$DEFINE ST4}
        {$IFNDEF VER120}
           {$IFNDEF VER125}
            {$DEFINE ST5}
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

interface

uses
  Windows, Graphics, StdCtrls, Classes, Messages, Controls, SysUtils;

const
  StdColors: array [0..15] of TColor =
  (clBlack, clMaroon, clGreen, clOlive,
  clNavy, clPurple, clTeal, clGray,
  clSilver, clRed, clLime, clYellow,
  clBlue, clFuchsia, clAqua, clWhite);

  SysColors: array [0..24] of TColor =
  (clScrollBar, clBackground, clActiveCaption,
  clInactiveCaption, clMenu, clWindow, clWindowFrame,
  clMenuText, clWindowText, clCaptionText, clActiveBorder,
  clInactiveBorder, clAppWorkSpace, clHighlight,
  clHighlightText, clBtnFace, clBtnShadow, clGrayText,
  clBtnText, clInactiveCaptionText, clBtnHighlight,
  cl3DDkShadow, cl3DLight, clInfoText, clInfoBk);


  ComponentVersion='1.11';
  ComponentAuthor='Jakub Dusek';
  ComponentEMail='j.dusek@seznam.cz';
  ComponentHomePage='http://dusek.zde.cz';

type
  TDFWSTColorsCombo = class (TCustomComboBox)
  private
    FColorNames		: TStringList;
    FColorHeight	: integer;
    FColorSelected	: TColor;
    FColorText		: string;
    FColorWidth		: integer;
    FDisplayNames	: Boolean;
    FSystemColors	: Boolean;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure SetColorNames (Value: TStringList);
    procedure SetColorHeight (Value: integer);
    procedure SetColorSelected (Value: TColor);
    procedure SetColorText (Value: string);
    procedure SetColorWidth (Value: integer);
    procedure SetDisplayNames (Value: Boolean);
    procedure SetSystemColors (Value: Boolean);
    procedure AddColors;
    procedure NamesChanged (Sender: TObject);
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize (var Message: TMessage); message WM_SIZE;
  protected
    procedure Change; override;
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create (AOwner:TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
    destructor Destroy; override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property ColorNames		: TStringList read FColorNames write SetColorNames;
    property ColorHeight	: integer read FColorHeight write SetColorHeight;
    property ColorSelected	: TColor read FColorSelected write SetColorSelected;
    property ColorText		: string read FColorText write SetColorText;
    property ColorWidth		: integer read FColorWidth write SetColorWidth default 22;
    property DisplayNames	: Boolean read FDisplayNames write SetDisplayNames default True;
    property SystemColors	: Boolean read FSystemColors write SetSystemColors default True;
    property Items;
   {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    {$ENDIF}
    {$IFDEF ST5}
    property OnContextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

  TDFWSTColorsList = class (TCustomListBox)
  private
    FColorNames		: TStringList;
    FColorHeight	: integer;
    FColorSelected	: TColor;
    FColorText		: string;
    FColorWidth		: integer;
    FDisplayNames	: Boolean;
    FSystemColors	: Boolean;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure SetColorNames (Value: TStringList);
    procedure SetColorHeight (Value: integer);
    procedure SetColorSelected (Value: TColor);
    procedure SetColorText (Value: string);
    procedure SetColorWidth (Value: integer);
    procedure SetDisplayNames (Value: Boolean);
    procedure SetSystemColors (Value: Boolean);
    procedure AddColors;
    procedure NamesChanged (Sender: TObject);
    procedure CMFontChanged (var Message: TMessage); message CM_FONTCHANGED;
    procedure WMSize (var Message: TMessage); message WM_SIZE;
  protected
    procedure Click; override;
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
  public
    constructor Create (AOwner:TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
    destructor Destroy; override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property ColorNames		: TStringList read FColorNames write SetColorNames;
    property ColorHeight	: integer read FColorHeight write SetColorHeight;
    property ColorSelected	: TColor read FColorSelected write SetColorSelected;
    property ColorText		: string read FColorText write SetColorText;
    property ColorWidth		: integer read FColorWidth write SetColorWidth default 22;
    property DisplayNames	: Boolean read FDisplayNames write SetDisplayNames default True;
    property SystemColors	: Boolean read FSystemColors write SetSystemColors default True;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    {$ENDIF}
    {$IFDEF ST5}
    property OnConTextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

implementation

function GetContrastColor (Color: TColor): TColor;
var r, g, b: Byte;
begin
  Color := ColorToRGB (Color);
  if GetRValue (Color) >= $80 then r := 0
  else
    r := $FF;
  if GetGValue (Color) >= $80 then g := 0
  else
    g := $FF;
  if GetRValue (Color) >= $80 then b := 0
  else
    b := $FF;
  Result := RGB (r,g,b);
end;

function GetItemHeight (Font: TFont): Integer;
var
  DC		: HDC;
  Fnt		: HFont;
  Metrics	: TTextMetric;
begin
  DC := GetDC (0);
  try
    Fnt := SelectObject (DC, Font.Handle);
    GetTextMetrics (DC, Metrics);
    SelectObject (DC, Fnt);
  finally
    ReleaseDC (0, DC);
  end;
  Result := Metrics.tmHeight + 1;
end;

procedure DrawColorItem (Index: integer; Canvas: TCanvas; Rect: TRect; Font: TFont;
                         DisplayNames: Boolean; ColorWidth: Integer;
                         Text,ColorText: string; Color: TColor; ColorNames: TStringList);
var r		: TRect;
    s,t		: string;
    c,fc	: TColor;
begin
  r := Rect;
  Canvas.FillRect (Rect);
  Inc (r.Top, 2);
  Inc (r.Left, 2);
  Dec (r.Bottom, 2);
  if not DisplayNames then Dec (r.Right, 2)
  else
    r.Right := r.Left + ColorWidth;
  c := Canvas.Brush.Color;
  Canvas.Brush.Color := Color;
  Canvas.Rectangle (R.Left, R.Top, R.Right, R.Bottom);
  s := Text;
  if ColorNames.Count < Index + 1 then
  begin
    if LowerCase (Copy (s, 1, 2)) = 'cl' then s := Copy (s, 3, Length (s));
  end
  else
  begin
    s := ColorNames [index];
  end;
  if ColorText <> '' then
  begin
    fc := Canvas.Font.Color;
    Canvas.Font.Color := GetContrastColor (Canvas.Brush.Color);
    t := ColorText;
    if t = '%1' then t := s;
    DrawText (Canvas.Handle, PChar(t), Length(t), R,
              DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX or DT_CENTER);
    Canvas.Font.Color := fc;
  end;
  Canvas.Brush.Color := c;
  rect.Left := r.Right + 2;
  DrawText (Canvas.Handle, PChar(s), Length(s), Rect,
            DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
end;

constructor TDFWSTColorsCombo.Create(AOwner:TComponent);
begin
  inherited Create (AOwner);
  Style := csOwnerDrawFixed;
  FColorWidth := 22;
  FColorHeight := ItemHeight;
  FDisplayNames := True;
  FSystemColors := True;
  FColorNames := TStringList.Create;
  TStringList (FColorNames).OnChange := NamesChanged;
end;

procedure TDFWSTColorsCombo.CreateWnd;
var i: integer;
begin
  i := GetItemHeight (Font);
  if i > FColorHeight then FColorHeight := i;
  Height := FColorHeight;
  ItemHeight := FColorHeight;
  inherited CreateWnd;
  AddColors;
end;

procedure TDFWSTColorsCombo.Loaded;
begin
  inherited Loaded;
  FColorHeight := ItemHeight;
  SetColorSelected (FColorSelected);
end;

destructor TDFWSTColorsCombo.Destroy;
begin
  FColorNames.Free;
  inherited Destroy;
end;

procedure TDFWSTColorsCombo.AddColors;
var i, Index	: integer;
    s		: string;
begin
  if Items.Count <> 0 then s := Items [ItemIndex];
  Items.Clear;
  for i := 0 to High (StdColors) do
  begin
    Items.AddObject (ColorToString (StdColors [i]), TObject (StdColors [i]));
  end;
  if FSystemColors then
  begin
    for i := 0 to High (SysColors) do
    begin
      Items.AddObject (ColorToString (SysColors [i]), TObject (SysColors [i]));
    end;
  end;
  Index := Items.IndexOf (s);
  if Index <> -1 then ItemIndex := Index
  else
    ItemIndex := 0;
  SetColorSelected (StringToColor (Items [ItemIndex]));
end;

procedure TDFWSTColorsCombo.SetColorNames (Value: TStringList);
begin
  FColorNames.Assign (Value);
  Invalidate;
end;

procedure TDFWSTColorsCombo.NamesChanged (Sender: TObject);
begin
  Invalidate;
end;

procedure TDFWSTColorsCombo.SetColorWidth (Value: integer);
begin
  FColorWidth := Value;
  Invalidate;
end;

procedure TDFWSTColorsCombo.SetSystemColors (Value: Boolean);
begin
  FSystemColors := Value;
  AddColors;
end;

procedure TDFWSTColorsCombo.SetColorHeight (Value: integer);
begin
  FColorHeight := Value;
  RecreateWnd;
end;

procedure TDFWSTColorsCombo.SetDisplayNames (Value: Boolean);
begin
  FDisplayNames := Value;
  Invalidate;
end;

procedure TDFWSTColorsCombo.SetColorSelected (Value: TColor);
var Index: Integer;
begin
  Index := Items.IndexOf (ColorToString (Value)); 
  if Index <> -1 then
  begin
    FColorSelected := Value;
    ItemIndex := Index;
    if not (csReading in ComponentState) and
       Assigned (OnChange) then OnChange (Self);
  end;
end;

procedure TDFWSTColorsCombo.SetColorText (Value: string);
begin
  FColorText := Value;
  Invalidate;
end;

procedure TDFWSTColorsCombo.CMFontChanged (var Message: TMessage);
begin
  inherited;
  RecreateWnd;
end;

procedure TDFWSTColorsCombo.WMSize (var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TDFWSTColorsCombo.Change;
begin
  SetColorSelected (StringToColor (Items [ItemIndex]));
  inherited Change;
end;

procedure TDFWSTColorsCombo.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  DrawColorItem (Index, Canvas,Rect, Font,FDisplayNames,
                 FColorWidth, Items[Index], FColorText, TColor (Items.Objects [Index]), ColorNames);
end;

constructor TDFWSTColorsList.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  Style := lbOwnerDrawFixed;
  FColorWidth := 22;
  FColorHeight := ItemHeight;
  FDisplayNames := True;
  FSystemColors := True;
  FColorNames := TStringList.Create;
  TStringList (FColorNames).OnChange := NamesChanged;
end;

procedure TDFWSTColorsList.CreateWnd;
var i: integer;
begin
  i := GetItemHeight (Font);
  if i > FColorHeight then FColorHeight := i;
  ItemHeight := FColorHeight;
  inherited CreateWnd;
  AddColors;
end;

procedure TDFWSTColorsList.Loaded;
begin
  inherited Loaded;
  FColorHeight := ItemHeight;
  SetColorSelected (FColorSelected);
end;

destructor TDFWSTColorsList.Destroy;
begin
  FColorNames.Free;
  inherited Destroy;
end;

procedure TDFWSTColorsList.AddColors;
var i, Index	: integer;
    s		: string;
begin
  if Items.Count <> 0 then s := Items [ItemIndex];
  Items.Clear;
  for i := 0 to High (StdColors) do
  begin
    Items.AddObject (ColorToString (StdColors [i]), TObject (StdColors [i]));
  end;
  if FSystemColors then
  begin
    for i := 0 to High (SysColors) do
    begin
      Items.AddObject (ColorToString (SysColors [i]), TObject (SysColors [i]));
    end;
  end;
  Index := Items.indexof(s);
  if Index <> -1 then ItemIndex := Index
  else
    ItemIndex := 0;
  SetColorSelected (StringToColor (Items [ItemIndex]));
end;

procedure TDFWSTColorsList.SetColorNames (Value: TStringList);
begin
  FColorNames.Assign (Value);
  Invalidate;
end;

procedure TDFWSTColorsList.NamesChanged (Sender: TObject);
begin
  Invalidate;
end;

procedure TDFWSTColorsList.SetColorWidth (Value: integer);
begin
  FColorWidth := Value;
  Invalidate;
end;

procedure TDFWSTColorsList.SetSystemColors (Value: Boolean);
begin
  FSystemColors := Value;
  AddColors;
end;

procedure TDFWSTColorsList.SetColorHeight (Value: integer);
begin
  FColorHeight := Value;
  RecreateWnd;
end;

procedure TDFWSTColorsList.SetDisplayNames (Value: Boolean);
begin
  FDisplayNames := Value;
  Invalidate;
end;

procedure TDFWSTColorsList.SetColorSelected (Value: TColor);
var Index: Integer;
begin
  Index := Items.IndexOf (ColorToString (Value)); 
  if Index <> -1 then
  begin
    FColorSelected := Value;
    ItemIndex := Index;
    if not (csReading in ComponentState) and
       Assigned (OnClick) then OnClick (Self);
  end;
end;

procedure TDFWSTColorsList.SetColorText (Value: string);
begin
  FColorText := Value;
  Invalidate;
end;

procedure TDFWSTColorsList.CMFontChanged (var Message: TMessage);
begin
  inherited;
  RecreateWnd;
end;

procedure TDFWSTColorsList.WMSize (var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TDFWSTColorsList.Click;
begin
  SetColorSelected (StringToColor (Items [ItemIndex]));
  inherited Click;
end;

procedure TDFWSTColorsList.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  DrawColorItem (Index, Canvas, Rect, Font, FDisplayNames,
                 FColorWidth, Items[Index], FColorText, TColor (Items.Objects [Index]), ColorNames);
end;

function TDFWSTColorsCombo.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSTColorsCombo.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSTColorsCombo.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSTColorsCombo.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

function TDFWSTColorsList.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSTColorsList.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSTColorsList.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSTColorsList.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

end.