{******************************************************************************}
{          This unit is part of the Delphifreeware component package.          }                                                                              
{                                                                              }
{ Filename           : DFWSTDrives.pas                                         }
{ Short description  : Two Delphi components that displays drives combo box or }
{                      list box with associated icons.                         }
{ Author             : Jakub Dusek                                             }
{ E-Mail             : j.dusek@seznam.cz                                       }
{ Internet           : http://dusek.zde.cz                                     }
{ Version            : 1.1                                                     }
{ Date last modified : 20.08.1999                                              }
{ Delphi versions    :                                                         }
{                                                                              }
{                                                                              }
{ Add. Copyright     :                                                         }
{                                                                              }
{ Description        :                                                         }
{                                                                              }
{ Usage              :                                                         }
{                                                                              }
{ Revision history   :                                                         }
{                                                                              }
{ This unit contains the following                                             }
{ Components       : TDFWSTDrivesCombo, TDFWSTDrivesList                       }
{ Classes          :                                                           }
{ Types            : TDriveType, TDrivesType, TIconSize, TTextCase             }
{ Constants        :                                                           }
{ Global variables :                                                           }
{ Global procedures:                                                           }
{ Global functions :                                                           }
{                                                                              }
{******************************************************************************}
{                                                                              }
{  DFW and Delphifreeware   2000, Carsten Butzek E-Mail: cabutzek@muenster.de }
{                                                                              }
{  Delphifreeware or DFW is a free Delphi component collection.                }
{  If you wrote a component which should be integrated into the DFW please     }
{  contact me.                                                                 }
{                                                                              }
{  Updates of the DFW package are available at the DFW homepage:               }
{  http://www.delphifreeware.de                                                }
{                                                                              }
{******************************************************************************}
{ License agreement                                                            }
{                                                                              }
{  This unit is freeware.                                                      }
{                                                                              }
{  Please do not release modified versions of this source code.                }
{  If you've made any changes that you think should have been there,           }
{  feel free to submit them to me at cabutzek@muenster.de                      }
{                                                                              }
{                                                                              }
{  This software is provided 'as-is', without any express or                   }
{  implied warranty. In no event will the author be held                       }
{  liable for any damages arising from the use of this                         }
{  software.                                                                   }
{  Permission is granted to anyone to use this software for                    }
{  any purpose, including commercial applications, and to                      }
{  alter it and redistribute it freely, subject to the                         }
{  following restrictions:                                                     }
{  1. The origin of this software must not be misrepresented,                  }
{     you must not claim that you wrote the original software.                 }
{     If you use this software in a product, an acknowledgment                 }
{     in the product documentation would be appreciated but is                 }
{     not required.                                                            }
{  2. Altered source versions must be plainly marked as such,                  }
{     and must not be misrepresented as being the original                     }
{     software.                                                                }
{  3. This notice may not be removed or altered from any                       }
{     source distribution.                                                     }
{  4. You must credit the DFW team in  your software including a               }
{     link to the DFW homepage at http://www.delphifreeware.de.                }
{                                                                              }
{ End of license agreement                                                     }
{                                                                              }
{******************************************************************************}

{$I DFW.INC}

unit DFWSTDrives;

{$IFNDEF VER80}
 {$IFNDEF VER90}
  {$IFNDEF VER93}
    {$IFNDEF VER100}
      {$IFNDEF VER110}
        {$DEFINE ST4}
        {$IFNDEF VER120}
           {$IFNDEF VER125}
            {$DEFINE ST5}
          {$ENDIF}
        {$ENDIF}
      {$ENDIF}
    {$ENDIF}
  {$ENDIF}
 {$ENDIF}
{$ENDIF}

interface

uses
  Windows, StdCtrls, Messages, Controls, Classes, ShellApi,
  Graphics, SysUtils {$IFDEF ST4}, ImgList {$ENDIF}
//---
,forms
  ;

const
      ComponentVersion='1.1';
      ComponentAuthor='Jakub Dusek';
      ComponentEMail='j.dusek@seznam.cz';
      ComponentHomePage='http://dusek.zde.cz';

type
//---
  TMessageEvent = procedure (var Msg: TMsg; var Handled: Boolean) of object;

  TDriveType = (dtUnkNown, dtNonExisting, dtRemovable,
                dtFixed, dtRemote, dtCDROM, dtRamDisk);
  
  TDrivesType = set of TDriveType;

  TIconSize = (isSmall, isLarge);

  TTextCase = (tcLowerCase, tcUpperCase);

  TDFWSTDrivesCombo = class (TCustomComboBox)
  private
    FAutoUpdate		: Boolean;
    FDisplayBrackets	: Boolean;
    FDisplayIcons	: Boolean;
    FDisplayLabel	: Boolean;
    FDisplaySysName	: Boolean;
    FDrive		: Char;
    FIconsSize		: TIconSize;
    FTextCase		: TTextCase;
    FDrivesType		: TDrivesType;

    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;

    procedure WndProc(var Msg: TMessage); override;

    procedure SetDisplayBrackets (Value: Boolean);
    procedure SetDisplayIcons (Value: Boolean);
    procedure SetDisplayLabel (Value: Boolean);
    procedure SetDisplaySysName (Value: Boolean);
    procedure SetDrive (Value: Char);
    procedure SetCase (Value: TTextCase);
    procedure SetIconsSize (Value: TIconSize);
    procedure SetDrivesType (Value: TDrivesType);
    procedure CMFontChanged (var Message: TMessage); Message CM_FONTCHANGED;
  protected
    procedure DropDown; override;
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Change; override;

  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property AutoUpdate		: Boolean read FAutoUpdate write FAutoUpdate default True;
    property DisplayBrackets	: Boolean read FDisplayBrackets write SetDisplayBrackets default True;
    property DisplayIcons	: Boolean read FDisplayIcons write SetDisplayIcons default True;
    property DisplayLabel	: Boolean read FDisplayLabel write SetDisplaylabel default True;
    property DisplaySysName	: Boolean read FDisplaySysName write SetDisplaySysName default True;
    property Drive		: Char read FDrive write SetDrive;
    property IconsSize		: TIconSize read FIconsSize write SetIconsSize;
    property TextCase		: TTextCase read FTextCase write SetCase default tcLowerCase;
    property DrivesType		: TDrivesType read FDrivesType write SetDrivesType default [dtRemovable,dtFixed, dtRemote, dtCDROM, dtRamDisk];
    procedure UpdateDrives;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    {$ENDIF}
    {$IFDEF ST5}
    property OnConTextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property DropDownCount;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnDropDown;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

  TDFWSTDrivesList = class (TCustomListBox)
  private
    FDisplayBrackets	: Boolean;
    FDisplayIcons	: Boolean;
    FDisplayLabel	: Boolean;
    FDisplaySysName	: Boolean;
    FDrive		: Char;
    FIconsSize		: TIconSize;
    FTextCase		: TTextCase;
    FDrivesType		: TDrivesType;
    function GetVersion:string;
    function GetAuthor:string;
    function GetEMail:string;
    function GetHomepage:string;
    procedure SetDisplayBrackets (Value: Boolean);
    procedure SetDisplayIcons (Value: Boolean);
    procedure SetDisplayLabel (Value: Boolean);
    procedure SetDisplaySysName (Value: Boolean);
    procedure SetDrive (Value: Char);
    procedure SetCase (Value: TTextCase);
    procedure SetIconsSize (Value: TIconSize);
    procedure SetDrivesType (Value:TDrivesType);
    procedure CMFontChanged (var Message: TMessage); Message CM_FONTCHANGED;
  protected
    procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
    procedure Click; override;
  public
    constructor Create (AOwner: TComponent); override;
    procedure CreateWnd; override;
    procedure Loaded; override;
  published
    property Version: string read GetVersion;
    property Author: string read GetAuthor;
    property EMail: string read GetEMail;
    property Homepage: string read GetHomepage;
    property DisplayBrackets	: Boolean read FDisplayBrackets write SetDisplayBrackets default True;
    property DisplayIcons	: Boolean read FDisplayIcons write SetDisplayIcons default True;
    property DisplayLabel	: Boolean read FDisplayLabel write SetDisplaylabel default True;
    property DisplaySysName	: Boolean read FDisplaySysName write SetDisplaySysName default True;
    property Drive		: Char read FDrive write SetDrive;
    property IconsSize		: TIconSize read FIconsSize write SetIconsSize;
    property TextCase		: TTextCase read FTextCase write SetCase default tcLowerCase;
    property DrivesType		: TDrivesType read FDrivesType write SetDrivesType default [dtRemovable,dtFixed, dtRemote, dtCDROM, dtRamDisk];
    procedure UpdateDrives;
    {$IFDEF ST4}
    property Anchors;
    property BiDiMode;
    property Constraints;
    property DragKind;
    property ParentBiDiMode;
    property OnEndDock;
    property OnStartDock;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    {$ENDIF}
    {$IFDEF ST5}
    property OnConTextPopup;
    {$ENDIF}
    {$IFNDEF VER90}
    property ImeMode;
    property ImeName;
    {$ENDIF}
    property Align;
    property BorderStyle;
    property Color;
    property Columns;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property ExtendedSelect;
    property Font;
    property IntegralHeight;
    property MultiSelect;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnStartDrag;
  end;

var Larges, Smalls	: TImageList;
    SFI			: TSHFileInfo;
    SysIL		: uint;

implementation

function GetDriveType (Drive: Char): TDriveType;
begin
  Result := TDriveType (Windows.GetDriveType (PChar (Drive + ':\')));
end;

function GetVolumeLabel (Drive: Char): string;
var VolLab		: array [0..MAX_PATH] of Char;
    MaxLength,SysFlag	: DWord;
    OldErrorMode	: Integer;
begin
  OldErrorMode := SetErrorMode (SEM_FAILCRITICALERRORS);
  try
    if GetVolumeInformation (PChar (Drive + ':\'), VolLab, 255,
                       nil, MaxLength, SysFlag, nil, 255) then Result := VolLab;
  finally
    SetErrorMode (OldErrorMode);
  end;
end;

procedure GetLogicalDrivesList (List: TStrings; DrivesType: TDrivesType);
 var
   Num		: Integer;
   Bits		: set of 0..25;
   Drive	: Char;
 begin
   List.Clear;
   Integer (Bits) := Windows.GetLogicalDrives;
   for Num := 0 to 25 do
   begin
     Drive := Char (Num + Ord ('A'));
     if (Num in Bits) and (GetDriveType (Drive) in DrivesType) then 
          List.Add (Drive);
   end;
end;

procedure DrawAssociatedImage (s: string; Cnvs: TCanvas; x, y: Integer; Size: TIconSize);
var Info	: TSHFileInfo;
    SSize	: Integer;
begin
  SSize := SizeOf (TSHFileInfo);
  FillChar (Info, SSize, 0);
  SHGetFileInfo (PChar (s), 0, Info, SSize, SHGFI_ICON or SHGFI_SMALLICON);
  case Size of
    isSmall: Smalls.Draw (Cnvs, x, y, Info.iIcon);
    isLarge: Larges.Draw (Cnvs, x, y, Info.iIcon);
  end;
end;

procedure DrawDriveItem(Drive: string; Canvas: TCanvas; Rect: TRect; DisplayLabel,
                        DisplaySysName, DisplayBrackets, DisplayIcons: Boolean;
                        TextCase: TTextCase; IconsSize: TIconSize);
var DriveLabel, DriveName	: string;
    x, y, Size			: Integer;
    Info			: TSHFileInfo;

//---  Add
  RootPathName: String;
  VolumeName: array[0..255] of Char;
  VolumeSerialNumber: DWord;
  MaximumComponentLength: DWord;
  FileSystemFlags: DWord;
  FileSystemName: array[0..10] of Char;
  Len: Integer;
  OutStr: string;
//--- End of Add

begin
  DriveLabel := Drive + ': ';
  DriveName := Drive + ':\';
  if not DisplaySysName then
  begin
    if (DisplayLabel) and (GetDriveType (DriveLabel [1]) <> dtRemovable) then
    begin
      if DisplayBrackets then DriveLabel := DriveLabel + '[';
      DriveLabel := DriveLabel + GetVolumeLabel (DriveLabel [1]);
      if DisplayBrackets then DriveLabel := DriveLabel + ']';
    end;
    case TextCase of
      tcLowerCase: DriveLabel := LowerCase(DriveLabel);
      tcUpperCase: DriveLabel := UpperCase(DriveLabel);
    end;
  end
  else
  begin
    Size := SizeOf (TSHFileInfo);

    FillChar (Info, Size, 0);
//--- Add
    if (LowerCase(DriveName[1])<>'a') and (LowerCase(DriveName[1])<>'b') then
    begin
      RootPathName:=DriveName[1]+':\';
      if GetVolumeInformation(PChar(RootPathName) {input},
                              VolumeName {output},
                              SizeOf(VolumeName) {input},
                              @VolumeSerialNumber {output},
                              MaximumComponentLength {output},
                              FileSystemFlags {output},
                              FileSystemName {output},
                              SizeOf(FileSystemName) {input}) then
        if VolumeName<>'' then
        begin
          OutStr:='';
          for Len:=0 to 255 do
            if VolumeName[Len]<>#0 then
              OutStr:=OutStr+VolumeName[Len]
            else
              break;
          OutStr:=OutStr[1]+LowerCase(Copy(OutStr,2,Length(OutStr)-1))+' ('+UpperCase(Drive)+':)';
          DriveLabel:=OutStr;
        end
        else
          if GetLastError=21 then
            DriveLabel:='('+UpperCase(Drive)+':)'
          else
          begin
            SHGetFileInfo (PChar (DriveName), 0, Info, Size, SHGFI_DISPLAYNAME);
            DriveLabel := Info.szDisplayName;
          end
      else
      begin
        SHGetFileInfo (PChar (DriveName), 0, Info, Size, SHGFI_DISPLAYNAME);
        DriveLabel := Info.szDisplayName;
      end;
    end
    else
//--- End of add...
    begin
      SHGetFileInfo (PChar (DriveName), 0, Info, Size, SHGFI_DISPLAYNAME);
      DriveLabel := Info.szDisplayName;
    end;
  end;
  Canvas.FillRect (Rect);
  Rect.Top := Rect.Top + 1;
  Rect.Bottom := Rect.Bottom - 1;
  Rect.Left := Rect.Left + 2;
  if DisplayIcons then
  begin
    case IconsSize of
      isSmall:
      begin
        y := Rect.Top + (((Rect.Bottom - Rect.Top) - 16) div 2);
        x := 20;
      end;
      isLarge:
      begin
        y := Rect.Top + (((Rect.Bottom-Rect.Top) - 32) div 2);
        x := 36;
      end;
    end;
    DrawAssociatedImage (DriveName, Canvas, Rect.Left, y, IconsSize);
  end
  else
    x := 2;
  Rect.Left := Rect.Left + x;
  DrawText (Canvas.Handle, PChar (DriveLabel), Length (DriveLabel), Rect,
            DT_SINGLELINE or DT_VCENTER or DT_NOPREFIX);
end;

function GetItemHeight (Font: TFont): Integer;
var
  DC		: HDC;
  Fnt		: HFont;
  Metrics	: TTextMetric;
begin
  DC := GetDC (0);
  try
    Fnt := SelectObject (DC, Font.Handle);
    GeTTextMetrics (DC, Metrics);
    SelectObject (DC, Fnt);
  finally
    ReleaseDC (0, DC);
  end;
  Result := Metrics.tmHeight + 1;
end;

constructor TDFWSTDrivesCombo.Create (AOwner: TComponent);
begin
  inherited Create(AOwner);
  Style := csOwnerDrawFixed;
  FDrivesType := [dtRemovable,dtFixed, dtRemote, dtCDROM, dtRamDisk];
  AutoUpdate := True;
  DisplayIcons := True;
  DisplayBrackets := True;
  DisplayLabel := True;
  DisplaySysName := True;
end;

//--- When a media is to be removed, has been removed or inserted, this message is sent (See UNDU March 1998)
procedure TDFWSTDrivesCombo.WndProc(var Msg: TMessage);
begin
  if (Msg.Msg = WM_DEVICECHANGE) then
    UpdateDrives;
  inherited;
end;
//--- End of add

procedure TDFWSTDrivesCombo.CreateWnd;
begin
  inherited CreateWnd;
  UpdateDrives;
end;

procedure TDFWSTDrivesCombo.Loaded;
begin
  inherited Loaded;
  SetDrive (FDrive);
end;

procedure TDFWSTDrivesCombo.DropDown;
begin
  if FAutoUpdate then UpdateDrives;
  inherited DropDown;
end;

procedure TDFWSTDrivesCombo.SetIconsSize (Value: TIconSize);
begin
  FIconsSize := Value;
  case Value of
    isSmall:
    begin
      if GetItemHeight (Font) < 18 then ItemHeight := 18;
      Height := ItemHeight;
    end;
    isLarge:
    begin
      if GetItemHeight (Font) < 34 then ItemHeight := 34;
      Height := ItemHeight;
    end;
  end;
  RecreateWnd;
  Invalidate;
end;

procedure TDFWSTDrivesCombo.SetDisplayIcons (Value: Boolean);
begin
  FDisplayIcons := Value;
  Invalidate;
end;

procedure TDFWSTDrivesCombo.UpdateDrives;
var s		: string;
    Index	: Integer;
begin
  s := Items [ItemIndex];
  Items.Clear;
  GetLogicalDrivesList( Items, FDrivesType);
  if Items.Count <> 0 then
  begin
    Index := Items.IndexOf (s); 
    if Index <> - 1 then ItemIndex := Index
    else
      ItemIndex := 0;
    s := Items [ItemIndex];
    FDrive := s [1];
  end;
end;

procedure TDFWSTDrivesCombo.Change;
var s: string;
begin
  s := Items [ItemIndex];
  FDrive := s [1];
  inherited Change;
end;

procedure TDFWSTDrivesCombo.SetCase (Value: TTextCase);
begin
  FTextCase := Value;
  Invalidate;
end;

procedure TDFWSTDrivesCombo.SetDrivesType (Value: TDrivesType);
begin
  FDrivesType := Value;
  UpdateDrives;
end;

procedure TDFWSTDrivesCombo.SetDrive (Value: Char);
begin
  if Items.IndexOf (Value) <> - 1 then
  begin
    ItemIndex := Items.IndexOf (Value);
    FDrive := Value;
    if Assigned (OnChange) then OnChange (Self);
  end;
end;

procedure TDFWSTDrivesCombo.SetDisplayBrackets (Value: Boolean);
begin
  FDisplayBrackets := Value;
  Invalidate;
end;

procedure TDFWSTDrivesCombo.SetDisplayLabel (Value: Boolean);
begin
  FDisplayLabel := Value;
  Invalidate;
end;

procedure TDFWSTDrivesCombo.SetDisplaySysName (Value: Boolean);
begin
  FDisplaySysName := Value;
  Invalidate;
end;

procedure TDFWSTDrivesCombo.CMFontChanged (var Message: TMessage);
begin
  inherited;
  ItemHeight := GetItemHeight (Font);
  SetIconsSize (FIconsSize);
end;

procedure TDFWSTDrivesCombo.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  DrawDriveItem (Items [Index], Canvas, Rect, FDisplayLabel,
                 FDisplaySysName, FDisplayBrackets,
                 FDisplayIcons, FTextCase, FIconsSize);
end;

constructor TDFWSTDrivesList.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  Style := lbOwnerDrawFixed;
  FDrivesType := [dtRemovable, dtFixed, dtRemote, dtCDROM, dtRamDisk];
  DisplayIcons := True;
  DisplayBrackets := True;
  DisplayLabel := True;
  DisplaySysName := True;
end;

procedure TDFWSTDrivesList.CreateWnd;
begin
  inherited CreateWnd;
  UpdateDrives;
end;

procedure TDFWSTDrivesList.Loaded;
begin
  inherited Loaded;
  SetDrive (FDrive);
end;

procedure TDFWSTDrivesList.SetIconsSize (Value: TIconSize);
begin
  FIconsSize := Value;
  case Value of
    isSmall: if GetItemHeight (Font) < 18 then ItemHeight := 18;
    isLarge: if GetItemHeight (Font) < 34 then ItemHeight := 34;
  end;
  RecreateWnd;
  Invalidate;
end;

procedure TDFWSTDrivesList.SetDisplayIcons (Value: Boolean);
begin
  FDisplayIcons := Value;
  Invalidate;
end;

procedure TDFWSTDrivesList.UpdateDrives;
var s		: string;
    Index	: Integer;
begin
  s := Items [ItemIndex];
  Items.Clear;
  GetLogicalDrivesList( Items, FDrivesType);
  if Items.Count <> 0 then
  begin
    Index := Items.IndexOf (s); 
    if Index <> - 1 then ItemIndex := Index
    else
      ItemIndex := 0;
    s := Items [ItemIndex];
    FDrive := s [1];
  end;
end;

procedure TDFWSTDrivesList.Click;
var s: string;
begin
  s := Items [ItemIndex];
  FDrive := s [1];
  inherited Click;
end;

procedure TDFWSTDrivesList.SetCase (Value: TTextCase);
begin
  FTextCase := Value;
  Invalidate;
end;

procedure TDFWSTDrivesList.SetDrivesType (Value: TDrivesType);
begin
  FDrivesType := Value;
  UpdateDrives;
end;

procedure TDFWSTDrivesList.SetDrive (Value: Char);
begin
  if Items.Indexof (Value) <> - 1 then
  begin
    ItemIndex := Items.IndexOf (Value);
    FDrive := Value;
    if Assigned (OnClick) then OnClick (Self);
  end;
end;

procedure TDFWSTDrivesList.SetDisplayBrackets (Value: Boolean);
begin
  FDisplayBrackets := Value;
  Invalidate;
end;

procedure TDFWSTDrivesList.SetDisplayLabel (Value: Boolean);
begin
  FDisplayLabel := Value;
  Invalidate;
end;

procedure TDFWSTDrivesList.SetDisplaySysName (Value: Boolean);
begin
  FDisplaySysName := Value;
  Invalidate;
end;

procedure TDFWSTDrivesList.CMFontChanged (var Message: TMessage);
begin
  inherited;
  ItemHeight := GetItemHeight (Font);
  SetIconsSize (FIconsSize);
end;

procedure TDFWSTDrivesList.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  DrawDriveItem (Items [Index], Canvas, Rect, FDisplayLabel,
                 FDisplaySysName, FDisplayBrackets,
                 FDisplayIcons, FTextCase, FIconsSize);
end;

function TDFWSTDrivesCombo.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSTDrivesCombo.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSTDrivesCombo.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSTDrivesCombo.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

function TDFWSTDrivesList.GetVersion:string;
begin
     Result:=ComponentVersion;
end;

function TDFWSTDrivesList.GetAuthor:string;
begin
     Result:=ComponentAuthor;
end;

function TDFWSTDrivesList.GetEMail:string;
begin
     Result:=ComponentEMail;
end;

function TDFWSTDrivesList.GetHomepage:string;
begin
     Result:=ComponentHomePage;
end;

initialization
  Smalls := TImageList.Create (nil);
  SysIL := SHGetFileInfo ('', 0, SFI, SizeOf (SFI), SHGFI_SYSICONINDEX or SHGFI_SMALLICON);
  if SysIL <> 0 then 
  begin
    Smalls.Handle := SysIL;
    Smalls.ShareImages := True;
    Smalls.DrawingStyle := dsTransParent;
  end;
  Larges := TImageList.Create (nil);
  SysIL := SHGetFileInfo ('', 0, SFI, SizeOf(SFI), SHGFI_SYSICONINDEX or SHGFI_LARGEICON);
  if SysIL <> 0 then 
  begin
    Larges.Handle := SysIL;
    Larges.ShareImages := True;
    Larges.DrawingStyle := dsTransParent;
  end;

finalization
  Larges.Free;
  Smalls.Free;
end.
